#ifndef UTILS_H_INCLUDED
#define UTILS_H_INCLUDED

// fr Visual Studio
#if defined(_MSC_VER)
typedef unsigned __int8 uint8_t;
typedef signed __int8 int8_t;
typedef unsigned __int16 uint16_t;
typedef signed __int16 int16_t;
typedef unsigned __int32 uint32_t;
typedef signed __int32 int32_t;
#endif

// Mersenne Twister-Zufallsgenerator
class MTRand
{
public:
    MTRand(uint32_t seed = 0) { this->seed(seed); }

    void seed(uint32_t seed)
    {
        next = 0;
        mt[0] = seed;
        for(uint32_t i = 1; i != 624; ++i) mt[i] = static_cast<uint32_t>(69069ull * mt[i - 1] + 1);
    }

    uint32_t operator ()()
    {
        if(next == 624) generateNumbers(), next = 0;
        uint32_t x = mt[next++];
        x ^= x >> 11;
        x ^= (x << 7) & 0x9D2C5680;
        x ^= (x << 15) & 0xEFC60000;
        x ^= x >> 18;
        return x;
    }

private:
    void generateNumbers()
    {
        const uint32_t xorTab[] = { 0, 0x9908B0DF };
        for(uint32_t i = 0; i != 227; ++i) { const uint32_t x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF); mt[i] = mt[i + 397] ^ (x >> 1) ^ xorTab[x & 1]; }
        for(uint32_t i = 227; i != 624; ++i) { const uint32_t x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF); mt[i] = mt[i - 227] ^ (x >> 1) ^ xorTab[x & 1]; }
        const uint32_t x = (mt[623] & 0x80000000) | (mt[0] & 0x7FFFFFFF);
        mt[623] = mt[396] ^ (x >> 1) ^ xorTab[x & 1];
    }

    uint32_t mt[624];
    uint32_t next;
};

#endif
